<?php

/*
 * Plugin Name: WooCommerce ReciboPagos Gateway
 * Description: Permite pagos usando ReciboPagos
 * Author: Cristian Contreras
 * Version: 1.0.1
 *
 */


require_once 'vendor/autoload.php';
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\RequestException;

add_filter( 'woocommerce_payment_gateways', 'recibopagos_add_gateway_class' );
function recibopagos_add_gateway_class( $gateways ) {
	$gateways[] = 'WC_ReciboPagos_Gateway'; 
	return $gateways;
}

add_filter('woocommerce_get_return_url','override_return_url',10,2);

function override_return_url($return_url,$order){

    $oid = array(
        'order_id' => $order->get_id()
    );
    $url_extension = http_build_query($oid);
    return $return_url.'&'.$url_extension;;

  }

add_action( 'plugins_loaded', 'recibopagos_init_gateway_class' );
function recibopagos_init_gateway_class() {
 
	class WC_ReciboPagos_Gateway extends WC_Payment_Gateway {


 
 		/**
 		 * Class constructor
 		 */
 		public function __construct() {
            $this->id = 'recibopagos'; 
            $this->icon = "https://api.recibopagos.com/assets/wordpress-logo";
            $this->has_fields = false; 
            $this->method_title = 'ReciboPagos';
            $this->method_description = 'Configuración medio de pago ReciboPagos';
            
            $this->supports = array(
                'products'
            );
        
            $this->init_form_fields();
        
            $this->init_settings();
            $this->title = $this->get_option( 'title' );
            $this->description = $this->get_option( 'description' );
            $this->enabled = $this->get_option( 'enabled' );
            $this->apikey = $this->get_option( 'api_key' );
            $this->apisecret = $this->get_option( 'api_secret' );
            $this->Client = new \GuzzleHttp\Client();
            $this->baseURL = 'https://api.recibopagos.com/api/';
            $this->headers = [
                'Authorization' => 'Bearer ' . $this->get_option( 'api_key' ),        
                'Accept'        => 'application/json',
            ];

            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
            add_action( 'woocommerce_api_'. $this->id, array( $this, 'processData' ) );
 		}
 

 		public function init_form_fields(){
            $this->form_fields = array(
                'enabled' => array(
                    'title'       => 'Activa/Desactiva',
                    'label'       => 'Activar ReciboPagos',
                    'type'        => 'checkbox',
                    'description' => '',
                    'default'     => 'no'
                ),
                'title' => array(
                    'title'       => 'Título',
                    'type'        => 'text',
                    'description' => 'Controla el titulo del medio de pago en la pagina de checkout',
                    'default'     => 'Paga con ReciboPagos',
                    'desc_tip'    => true,
                ),
                'description' => array(
                    'title'       => 'Description',
                    'type'        => 'textarea',
                    'description' => 'Indica el descriptor del medio de pago en la pagina de checkout',
                    'default'     => 'Paga con ReciboPagos de manera segura',
                ),
                'api_key' => array(
                    'title'       => 'Token de autenticación',
                    'type'        => 'text',
                    'description' => 'Llave única de cada comercio',
                ),
                'api_secret' => array(
                    'title'       => 'Llave de cliente',
                    'type'        => 'text',
                    'description' => 'Secreto compartido entre recibopagos y la tienda',
                )


                
            );
        
        
 
        }
        private function generateSignature(Array $array){
            $r = $array;
            array_walk($r,
                            function (&$v, $k) {
                                $v = $k.$v;
                            }
                        );
            return implode("", $r);
        }
        
		public function process_payment( $order_id ) {
            global $woocommerce;
            
            $order = new WC_Order( $order_id );
            $items = $order->get_items();
            $product_list = '';

            foreach ($items as $item) {
                $product = $item->get_product();
                $product_name = $product->get_name();
                $product_quantity = $item->get_quantity();
                $product_list .= "Producto: $product_name - Cantidad: $product_quantity\n";
            }

            $paymentObject = array(
                'merchant_order_id'     => $order_id,
                'order_description'     => "Orden ".$order_id." Comercio ".get_bloginfo('name'),
                'client_email'          => $order->get_billing_email(),
                'notification_url'      => site_url('wc-api/recibopagos'),
                'return_url'            => $this->get_return_url( $order ),
                'currencies'            => array(
                                                'CLP' => number_format($order->get_total(), 0, '',''),
                                            ),
                'created_by'            => 'wordpress',
             ) ;

             ksort($paymentObject);
             $firma = $paymentObject;
             $firma['currencies'] = $this->generateSignature($firma['currencies']);
             $signature = $this->generateSignature($firma);
             
            try {
                $r = $this->Client->request('POST', $this->baseURL.'orders', ['headers' => $this->headers, 'json' => $paymentObject]);
                if($r->getStatusCode()==200){
                    $resJson = json_decode($r->getBody()->getContents());
                    $order->update_status('pending', 'Esperando confirmación de pago desde ReciboPagos');
                    $order->update_meta_data( 'order_id', $resJson->order_id );
                    $order->update_meta_data( 'enviroment', $resJson->enviroment );
                    $order->update_meta_data( 'status', $resJson->status );
                    $order->update_meta_data( 'checkout_url', $resJson->checkout_url );
                    $order->update_meta_data( 'payment_methods', $resJson->payment_methods );
                    $order->update_meta_data( 'order_hash', hash_hmac('sha256', $signature, $this->apisecret));
                    
                    $order->save();
                    return array(
                        'result' => 'success',
                        'redirect' => $resJson->checkout_url
                    );
                }
                else {
                    wc_add_notice($resJson->estado, 'error' );
                    error_log('No pudo procesarse la orden');
                }
            }
            catch(RequestException $e){
                $e->hasResponse() ? wc_add_notice(Psr7\str($e->getResponse()), 'error' ) : wc_add_notice('Ha ocurrido un error inesperado, favor intente nuevamente mas tarde', 'error' );
                return array(
                    'result' => 'failure',
                    'redirect' => ''
                );
            }
            
 
        }

        public function processData() {
            global $woocommerce;
            $decoded_data = json_decode(file_get_contents( 'php://input' ));
            if (isset($decoded_data->merchant_order_id) && isset($decoded_data->status)) {
                $order_id = $decoded_data->merchant_order_id;
                $status = $decoded_data->status;
                $order_hash =  $decoded_data->order_hash;
                $order = wc_get_order($order_id);
    
                if ($order) {
                    $order->update_meta_data( '_recibeNotificacion', 'SI' );
                    $order->update_meta_data( '_received_order_hash', $decoded_data->order_hash );
                    if ($status == 'paid' && $order_hash == $order->get_meta('order_hash')) {
                        $order->update_meta_data( '_fechaOperacion', $decoded_data->date );
                        $order->update_meta_data( '_horaOperacion', $decoded_data->time );
                        $order->update_meta_data( '_status', $decoded_data->status );
                        $order->update_meta_data( '_metodoPago', $decoded_data->payment_method );
                        $order->update_meta_data( '_montoTotal', $decoded_data->paid_amount );
                        $order->update_meta_data( '_moneda', $decoded_data->paid_currency );
                        
                        
                        $order->update_meta_data( '_mensaje', $decoded_data->message );
                        $order->update_status('processing');
                        $order->add_order_note( 'ReciboPagos Pay - Pago aprobado.'  );

                        header( 'HTTP/1.1 200' );
                    } elseif ($status == 'fail') {
                        $order->update_status('failed');
                        $order->update_meta_data( '_mensaje', $decoded_data->message );
                        header( 'HTTP/1.1 201 ERROR EN PAGO' );
                    }
                    else {
                        header( 'HTTP/1.1 203 RESPUESTA DESCONOCIDA' );
                    }

                    
                }
            }
            else {
                header( 'HTTP/1.1 500 Error procesando orden' );
            }
        
            
            
        }
         
 	}
}







add_action( 'woocommerce_thankyou', 'check_response', 1 );
function check_response($order_id) {
    global $woocommerce;
    $order = new WC_Order( $order_id );
    if($order->get_payment_method() == "recibopagos"){
        if($order->has_status('processing')) {
            $woocommerce->cart->empty_cart();
        }
        else {
            wp_safe_redirect( $order->get_checkout_payment_url() );
        }
    }
    return;
 }
